     ' ************************************************
     ' **  Name:          SEDITY2k.BAS               **
     ' **  Type:          Program                    **
     ' **  Module:        Main Module                **
     ' **  Language:      Microsoft QuickBASIC 4.00  **
     ' **  Programmer:    A. Eliason    08-16-93     **
     ' ************************************************
     '
     ' Main program for editing and archiving Coulter and RSA data files.
     ' Y2K revisions 10-10-98 AHE
     '
     ' USAGE:            No command line parameters
     ' .MAK FILE:        (none)
     ' PARAMETERS:       (none)
     ' ARRAYS:
     '    CLTlabNos$()
     '    RSAlabNos$()
     '    QPhiData$(0-9)      RSA data
     '    QVolPcnt$(0-15)     Cltr data - volume %
     '    QMicronDia$(0-15)   Cltr data - Micron Diameter
     '    StrLength%(18)      max allowed lenghth for input strings
     '
     '
     ' VARIABLES:
     '    A$                  temporary string input
     '    QLabNo$             full lab #
     '    QTubeDia$           tube diameter
     '    QSampID$            sample i.d.
     '    QProjID$            project i.d.
     '    QCruiseID$          cruise i.d.
     '    QReqBy$             requested by
     '    QOperID$            operator i.d.
     '    Y2K changed format to:
     '    QDate$              full date @ mmddyyyy
     '    QDate$  old format  full date @ mm/dd/yy
     '    QTime$              full time @ hh:mm:ss
     '    QSampWeight$        sample weight       RSA only
     '    QCoarseWeight$      coarse weight        "   "
     '    QSandWeight$        sand weight          "   "
     '
     '
     ' DECLARATIONS:
     DECLARE FUNCTION KeyCode% ()
     ' CONSTANTS:                          
     FALSE = 0
     TRUE = NOT FALSE
     ESC$ = CHR$(27)
     FormFeed$ = CHR$(12)



     ' *********************** PROGRAM SEDITY2K.BAS **************************
     ' Revised for run under QuickBasic V4.0      4/30/89     AHE
     ' Revised for larger buffer files            1/21/93     AHE
     ' This is the development version for all 1993 mods which include
     ' a. Elimination of line numbers
     ' b. Proper indentation of loops
     ' c. Renaming of Subroutines, variables and constants for clarity
     ' d. Elimination of old basic GOTO's where possible
     ' e. Check LAB.No. and change to upper case if nec.
     ' f. Modify so PgUp & PgDn keys can scroll through data faster.
     ' g. Check Master File Name for null entry
     ' h. Display/print Tube diameter - include in .NDX file
     ' Y2K Revised 10-10-98 to pass a date with four digit year to the Xfer file

     DEFINT I-M               ' this old Fortran convetion should be changed
     WIDTH LPRINT 255
     DIM RSAlabNos$(1000), CLTlabNos$(1000)       'Changed 600 to 1000 1-21-93 AHE
     DIM QMicronDia$(16), QVolPcnt$(16), QPhiData$(10), ED$(42)
     DIM StrLength%(18)
     DATA 6,5,3,16,16,16,16,16,8,8,7,7,7,5,5,5,5,5:   ' EDIT MAX $ LENGTHS
     FOR Index% = 1 TO 18
          READ StrLength%(Index%)
     NEXT Index%
     ' *********************** INTRODUCTION *********************************
 
     ' Initialize the display
     WIDTH 80
     CLS

Begin:

     PRINT "  ***************** SEDIT 93 **********************"
     PRINT "  U.S.G.S. Sed Lab  RSA and COULTER data editor "
     PRINT "  Eliason Data Services (508) 477-1400    06/17/93"
     PRINT "                Version 101098 Y2K"
     PRINT "  This program consists of utilities to inspect, edit, and archive data which"
     PRINT "  have been generated by the RSA and COULTER programs. In most cases, you will"
     PRINT "  need only to enter the letter or number requested by the program queries to"
     PRINT "  perform the desired action.  It will NOT be necessary to follow these entries"; : PRINT "  with the `Enter' key.  The `Enter' key is used to scroll to the next"
     PRINT "  item of a list or to terminate a correction in the EDIT mode."
     PRINT
     PRINT
     PRINT "Do you wish to display and/or edit:"
     PRINT "(R)     RSA Data "
     PRINT "(C)     COULTER Data "
     PRINT "(B)     Both"
     PRINT "?";

     DO
          DO
               A$ = INKEY$
          LOOP WHILE A$ = ""
          A$ = UCASE$(A$)
          PRINT A$;
          RSAflag% = FALSE
          CLTRflag% = FALSE
          Bothflag% = FALSE
         
          SELECT CASE A$

          CASE "R"
               RSAflag% = TRUE
          CASE "C"
               CLTRflag% = TRUE
          CASE "B"
               RSAflag% = TRUE
               CLTRflag% = TRUE
               Bothflag% = TRUE
          CASE ELSE
               PRINT "Please reenter your selection. R or C or B ? ";
          END SELECT
     LOOP WHILE NOT CLTRflag% AND NOT RSAflag%    ' non valid entry
     PRINT

     GOSUB ReadIndexFiles                     ' READ INDEX FILES INTO ARRAY

MainMenu:
DO
     '*********************** Main Menu ***************************

     PRINT FormFeed$;
     Prntr% = FALSE
     PRINT "Display / Edit Menu for ";
     IF RSAflag% THEN PRINT "RSA ";
     IF Bothflag% THEN PRINT "and ";
     IF CLTRflag% THEN PRINT "COULTER ";
     PRINT "Data"
     PRINT : PRINT : PRINT
     PRINT " 1. Display Status of logged samples"
     PRINT " 2. Print Status of logged samples"
     PRINT " 3. Assign Master File Number to logged samples"
     PRINT " 4. Edit / Display logged sample data"
     PRINT " 5. Archive Assigned samples to transfer disk"
     PRINT " 6. Begin (Restart program)"
     PRINT " 7. Exit to System (Terminate program)"

     DO
          PRINT : PRINT "Which number ?";
          DO
               A$ = INKEY$
          LOOP WHILE A$ = ""
          PRINT A$
          N% = VAL(A$)
     LOOP WHILE N% < 1 OR N% > 7

     SELECT CASE N%

          CASE 1                             ' Display Status
          Prntr% = FALSE
          GOSUB Status

          CASE 2                             ' Print Status
          Prntr% = TRUE
          ON ERROR GOTO PrintFault
          LPRINT CHR$(0);
          ON ERROR GOTO 0
          GOSUB Status

          CASE 3                             ' Assign
          PRINT FormFeed$;
          DO
               INPUT "Enter new Master File name to be assigned ", MFN$
               IF LEN(MFN$) > 6 THEN
                    BEEP
                    PRINT "NAME TOO LONG. 6 CHARACTERS PLEASE"
                    PRINT
               END IF
               IF MFN$ = "" THEN                        'SEDIT 93 AHE
                    BEEP
                    PRINT "PLEASE ENTER MFN"
               END IF
          LOOP WHILE LEN(MFN$) > 6 OR MFN$ = ""         'SEDIT 93 AHE
          GOSUB EditAssign

          CASE 4                             ' Edit
          MFN$ = "??????"
          PRINT FormFeed$;
          GOSUB EditAssign

          CASE 5                             ' Archive
          GOSUB Archive

          CASE 6                             ' Restart
          GOTO Begin

          CASE 7                             ' Quit
          CLOSE
          CLS
          PRINT : PRINT : BEEP
          PRINT "All files closed. Returning to System."
          SLEEP 2
          SYSTEM

          CASE ELSE
          END SELECT
LOOP                                         ' Main Menu Again

' ******************** End of Main Program **********************************

' ***************************
' ***  Major Subroutines  ***
' ***************************

Status:
'************ DISPLAY OR PRINT STATUS OF LOGGED SAMPLES ******************

     FLN$ = "CLTR": IF RSAflag% THEN FLN$ = "RSA"
     K% = 1
     PRINT FormFeed$;
     PRINT "   STATUS OF LOGGED SAMPLES"
     PRINT
     PRINT "Type  Lab Number Status"
     PRINT
     IF Prntr% THEN LPRINT "   STATUS OF LOGGED SAMPLES"
     IF Prntr% THEN
          LPRINT
          LPRINT "Type  Lab Number Status"
          LPRINT
     END IF
     DO
          N% = Cfiles%: IF FLN$ = "RSA" THEN N% = Rfiles%
          IF FLN$ = "RSA" THEN GOSUB OpenRSA      ' OPEN RSA
          IF FLN$ = "CLTR" THEN GOSUB OpenCLTR    ' OPEN CLTR.DAT
          FOR Index% = 1 TO N%
               K% = K% + 1
               GOSUB DisplayStatus           ' DISPLAY/PRINT Index%th record
               'PRINT
               IF NOT Prntr% AND K% MOD 22 = 0 THEN
                    PRINT
                    PRINT "Hit `Enter' for more or `Q' or `ESC' to Quit."
                    DO
                         B$ = INKEY$
                    LOOP WHILE B$ = ""
               END IF
               IF B$ = "Q" OR B$ = "q" OR B$ = ESC$ THEN EXIT FOR
          NEXT Index%
          CLOSE
          DoAgain% = FALSE
          IF Bothflag% AND FLN$ = "RSA" THEN
               FLN$ = "CLTR"
               DoAgain% = TRUE
          END IF
     LOOP WHILE DoAgain% AND B$ <> "Q" AND B$ <> "q" AND B$ <> ESC$
     IF Prntr% THEN LPRINT FormFeed$;
     INPUT "End of data. Hit `Enter' to return to menu. ", B$
RETURN


EditAssign:
'*************************** EDIT / ASSIGN ***********************************
     PRINT "Edit / Examine Mode    Master File Name: "; MFN$
     PRINT
     GOSUB EditMenu:                         ' PRINT MENU
     PRINT
     IF RSAflag% THEN
          K% = 1
          N% = Rfiles%
          FLN$ = "RSA"
          IC% = FALSE
          GOSUB OpenRSA                      ' OPEN RSA.DAT
     ELSE
          K% = 1
          N% = Cfiles%
          FLN$ = "CLTR"
          IC% = TRUE
          GOSUB OpenCLTR                     ' OPEN CLTR.DAT
     END IF

Redux:
     Index% = K%:                      ' ***** RE-ENTRY POINT *****
'PRINT K%; "."; CHR$(9); :
GOSUB DisplayStatus                    ' DISPLAY STATUS
PRINT "  ";

        kee% = KeyCode%         ' SEDIT 93 AHE

SELECT CASE kee%
        CASE 18432, 19200               ' left or up arrow
        A$ = "B"

        CASE 19712, 20480               ' right or down arrow
        A$ = CHR$(13)

        CASE 18688                      ' page up
        A$ = "PU"

        CASE 20736                      ' page down
        A$ = "PD"
       
        CASE ELSE
                IF kee% > 255 THEN
                        A$ = "X"
                ELSE
                        A$ = CHR$(kee%)
                END IF
END SELECT


A$ = UCASE$(A$)
IF A$ <> CHR$(13) THEN PRINT A$;
IF A$ = "/" OR A$ = "?" OR A$ = "H" THEN
     PRINT
     GOSUB EditMenu
     GOTO Redux
END IF

IF A$ = "A" THEN
     IF MFN$ = "??????" THEN
          PRINT "No Master File name assigned."
          BEEP
          DO
                    INPUT "Enter new Master File name to be assigned ", MFN$
                    IF LEN(MFN$) > 6 THEN
                         BEEP
                         PRINT "NAME TOO LONG. 6 CHARACTERS PLEASE"
                         PRINT
                    END IF
          LOOP WHILE LEN(MFN$) > 6
     END IF
     GOSUB IsItAssigned:   ' TEST FOR PREVIOUS ASSIGNMENT ETC
     IF NoGood% THEN GOTO Redux
     IF NOT IC% THEN RSAlabNos$(K%) = LEFT$(RSAlabNos$(K%), 5) + "*" + C$
     IF IC% THEN CLTlabNos$(K%) = LEFT$(CLTlabNos$(K%), 5) + "*" + C$
     GOTO Redux
END IF

IF A$ = "B" THEN
     IF K% = 1 THEN K% = N%: GOTO Redux
     K% = K% - 1: GOTO Redux
END IF

IF A$ = "D" THEN
     GOSUB IsItAssigned
     IF NoGood% THEN GOTO Redux
     IF NOT IC% THEN RSAlabNos$(K%) = LEFT$(RSAlabNos$(K%), 5) + "\" + C$
     IF IC% THEN CLTlabNos$(K%) = LEFT$(CLTlabNos$(K%), 5) + "\" + C$
     GOTO Redux
END IF

IF A$ = "E" THEN
     GOSUB Edit
     GOTO Redux
END IF

IF A$ = "O" THEN
     GOSUB IsItAssigned
     IF NoGood% THEN GOTO Redux
     IF NOT IC% THEN RSAlabNos$(K%) = LEFT$(RSAlabNos$(K%), 5) + "^" + C$
     IF IC% THEN CLTlabNos$(K%) = LEFT$(CLTlabNos$(K%), 5) + "^" + C$
     GOTO Redux
END IF

IF A$ = CHR$(13) THEN
     K% = K% + 1
     IF K% > N% AND Bothflag% AND NOT IC% THEN
          K% = 1
          N% = Cfiles%
          FLN$ = "CLTR"
          IC% = TRUE
          GOSUB OpenCLTR                     ' OPEN CLTR.DAT
          GOTO Redux
     END IF

     IF K% > N% AND Bothflag% AND IC% THEN
          K% = 1
          N% = Rfiles%
          FLN$ = "RSA"
          IC% = FALSE
          GOSUB OpenRSA                      ' OPEN RSA.DAT
          GOTO Redux
     END IF

     IF K% > N% THEN
          K% = 1
          GOTO Redux
     END IF
     GOTO Redux
END IF

IF A$ = "PD" THEN
     StartLine% = K%
     FinishLine% = K% + 17
     FOR LineNo% = StartLine% TO FinishLine%
     
             K% = K% + 1
             IF K% > N% AND Bothflag% AND NOT IC% THEN
                  K% = 1
                  N% = Cfiles%
                  FLN$ = "CLTR"
                  IC% = TRUE
                  GOSUB OpenCLTR                     ' OPEN CLTR.DAT
             END IF

             IF K% > N% AND Bothflag% AND IC% THEN
                  K% = 1
                  N% = Rfiles%
                  FLN$ = "RSA"
                  IC% = FALSE
                  GOSUB OpenRSA                      ' OPEN RSA.DAT
             END IF

             IF K% > N% THEN
                  K% = 1
             END IF
            
             Index% = K%
             GOSUB DisplayStatus

     NEXT LineNo%
     K% = K% + 1
     GOTO Redux
END IF

IF A$ = "PU" THEN
     K% = K% - 20
     IF K% <= 1 THEN K% = N%
     GOTO Redux
END IF


IF A$ <> "Q" AND A$ <> ESC$ THEN
     PRINT
     PRINT " What?"
     BEEP
     GOTO Redux
END IF

' At this point the command must have been "Q" or "ESC" to quit
PRINT
PRINT "DOING ASSIGNMENTS..."
SLEEP 2
' ***** DO THE ASSIGNMENTS AND DELETIONS *****
CLOSE
IF RSAflag% THEN
     GOSUB OpenRSA                      ' OPEN RSA.DAT
     OPEN "R", #2, "RSA.NDX", 9         ' AHE 8-93
     FIELD #2, 9 AS DISC$
     FOR K% = 1 TO Rfiles%
          A$ = LEFT$(RSAlabNos$(K%), 5)
          B$ = MID$(RSAlabNos$(K%), 6, 1)
          C$ = RIGHT$(RSAlabNos$(K%), 3)

          IF B$ = "\" THEN
               RSAlabNos$(K%) = A$ + "D" + C$
               LSET DISC$ = A$ + "D" + C$
               PUT #2, K% + 1
          END IF

          IF B$ = "^" THEN
               RSAlabNos$(K%) = A$ + "O" + C$
               LSET DISC$ = A$ + "O" + C$
               PUT #2, K% + 1
          END IF

          IF B$ = "*" THEN
               GET #1, K%
               RSAlabNos$(K%) = A$ + "A" + C$
               LSET QM$ = MFN$
               LSET DISC$ = A$ + "A" + C$
               PUT #1, K%
               PUT #2, K% + 1
          END IF
     NEXT K%

     CLOSE
END IF

IF CLTRflag% THEN
     GOSUB OpenCLTR                          ' OPEN CLTR.DAT
     OPEN "R", #2, "CLTR.NDX", 9
     FIELD #2, 9 AS DISC$
     FOR K% = 1 TO Cfiles%
          A$ = LEFT$(CLTlabNos$(K%), 5)
          B$ = MID$(CLTlabNos$(K%), 6, 1)
          C$ = RIGHT$(CLTlabNos$(K%), 3)
          IF B$ = "\" THEN
               CLTlabNos$(K%) = A$ + "D" + C$
               LSET DISC$ = A$ + "D" + C$
               PUT #2, K% + 1
          END IF

          IF B$ = "^" THEN
               CLTlabNos$(K%) = A$ + "O" + C$
               LSET DISC$ = A$ + "O" + C$
               PUT #2, K% + 1
          END IF

          IF B$ = "*" THEN
               GET #1, K%
               CLTlabNos$(K%) = A$ + "A" + C$
               LSET QM$ = MFN$
               LSET DISC$ = A$ + "A" + C$
               PUT #1, K%
               PUT #2, K% + 1
          END IF
     NEXT K%
     CLOSE
END IF
RETURN
'GOTO MainMenu



Edit:
'****************************** EDIT... *************************************
     GET #1, K%
     IF NOT IC% THEN GOSUB FillBufRSA:            ' FILL ED BUFFER RSA
     IF IC% THEN GOSUB FillBufCLTR:   '           ' Fill Ed Buffer CLTR
     IF NOT IC% THEN M = 23: KE% = 18:            ' BUFFER SIZE & ED LIMIT RSA
     IF IC% THEN M = 42: KE% = 10:                ' Buffer size & ED Limit CLTR
DO
     PRINT FormFeed$; :   ' HOME
     PRINT "******************** SED LAB EDIT  ********************"
     FOR Index% = 1 TO KE%
          PRINT Index%; " "; ED$(Index%)
     NEXT Index%
     PRINT
     PRINT "ENTER:   Line # to edit   P to Print   V to View data   Q or ESC to Quit";
     DO
          A$ = INKEY$
     LOOP WHILE A$ = ""
     IF A$ = ESC$ THEN A$ = "Q"
     IF VAL(A$) > 0 THEN
          PRINT A$;
          INPUT "", C$
          A$ = A$ + C$
     END IF
     A$ = UCASE$(A$)
    
     SELECT CASE A$
    
     CASE "P"
     
     LPRINT "SEDIT LISTING     "; FLN$; ".DAT    RECORD # "; K%
     LPRINT
     FOR Index% = 1 TO M
          LPRINT Index%; " "; ED$(Index%)
     NEXT Index%
     LPRINT FormFeed$;

     CASE "V"
    
     PRINT FormFeed$;
     PRINT "Data from "; ED$(1); "  "; ED$(2); "  type: "; ED$(3)
     PRINT " "; ED$(4); " "; ED$(5); " "; ED$(6)
     PRINT " "; ED$(7); " "; ED$(8)
     PRINT " "; ED$(9); " "; ED$(10): PRINT
     IK% = 23: IF IC% THEN IK% = 26
     FOR Index% = 11 TO IK%
          PRINT " "; ED$(Index%);
          IF IC% THEN PRINT " "; ED$(Index% + 16);
          PRINT
     NEXT Index%
     PRINT
     INPUT "Hit `Enter' to return to EDIT mode. ", A$
     
     CASE "Q"

     PRINT FormFeed$; "Edit / Examine Mode   Master File Name: "; MFN$: PRINT
     GOSUB EditMenu                          ' EDIT MENU
     EXIT DO                                 ' The only way out !!!!!!
                                             ' RE-ENTER EDIT/ASSIGN

     CASE ELSE
     IK% = VAL(A$)
          IKError = FALSE
          IF IK% < 1 OR IK% > KE% THEN
               BEEP
               PRINT "Nunber out of range or bad command."
               IKError% = TRUE
          END IF
         
          IF IK% = 1 AND ED$(1) = "      " THEN
               PRINT "Master File name not assigned. Use ASSIGN function."
               BEEP
               IKError% = TRUE
          END IF

          IF IK% = 3 AND NOT IC% THEN
               PRINT "RSA can not be wrong!"
               BEEP
               IKError% = TRUE
          END IF
          
          IF NOT IKError% THEN
TryAgain:
               PRINT "Change "; ED$(IK%); " to what? ";
               INPUT A$
               IF IK% = 3 AND A$ <> "30" AND A$ <> "200" THEN
               PRINT "Only 30 or 200 acceptable here!": GOTO TryAgain
               END IF

               IF LEN(A$) > StrLength%(IK%) THEN
               PRINT "Entry too long. Must be "; StrLength%(IK%); " characters maximum."
               GOTO TryAgain
               END IF

               IF IK% = 2 THEN A$ = UCASE$(A$)
               IF IK% = 2 AND LEN(A$) < 5 THEN
               PRINT "Lab Number must be 5 characters e.g. AE345"
               GOTO TryAgain
               END IF

               ED$(IK%) = A$
               IF NOT IC% THEN GOSUB SetRSA:   ' SET RSA.DAT
               IF IC% THEN GOSUB SetCLTR:   '     SET CLTR>DAT
               PUT #1, K%
              
               IF IK% = 2 THEN
               ' **** SPECIAL CASE - MUST UPDATE .NDX FILE ****
                    OPEN "R", #2, FLN$ + ".NDX", 9
                    FIELD #2, 9 AS DISC$
                    GET #2, K% + 1
                    ST1$ = MID$(DISC$, 6, 4)
                    LSET DISC$ = A$ + ST1$
                    PUT #2, K% + 1
                    CLOSE #2
                    IF NOT IC% THEN
                         RSAlabNos$(K%) = A$ + RIGHT$(RSAlabNos$(K%), 4)
                    ELSE
                         CLTlabNos$(K%) = A$ + RIGHT$(CLTlabNos$(K%), 4)
                    END IF
               END IF
               IF IK% = 3 THEN
               ' **** SPECIAL CASE - MUST UPDATE .NDX FILE ****
                    IF A$ = "30" THEN A$ = " 30"
                    OPEN "R", #2, FLN$ + ".NDX", 9
                    FIELD #2, 9 AS DISC$
                    GET #2, K% + 1
                    ST1$ = LEFT$(DISC$, 6)
                    LSET DISC$ = ST1$ + A$
                    PUT #2, K% + 1
                    CLOSE #2
                    IF NOT IC% THEN
                         RSAlabNos$(K%) = LEFT$(RSAlabNos$(K%), 6) + A$
                    ELSE
                         CLTlabNos$(K%) = LEFT$(CLTlabNos$(K%), 6) + A$
                    END IF
               END IF
         
          END IF
     END SELECT
LOOP
RETURN

Archive:
'****************** ARCHIVE (Write assigned data to transfer disk) **************
     PRINT FormFeed$; :   ' HOME
     PRINT "********************************* ARCHIVE *************************************"
     PRINT
     PRINT "This program segment is used to write ASSIGNED data records to the Transfer"
     PRINT "file/disk. This routine must be used with great care due to the following "
     PRINT "considerations:"
     PRINT
     PRINT "1. The old transfer file will be erased, NOT appended, so be sure that it"
     PRINT " has been put in the Master computer system. As a measure of security the"
     PRINT " file SEDXFER.DAT will be renamed SEDXFER.OLD before a new SEDXFER.DAT is "
     PRINT " written. (SEDIT93 version) "
     PRINT "2. When ARCHIVE finds that all data have been filed or deleted it will"
     PRINT " allow erasure the temporary data files on the RSA/COULTER disk."
     PRINT "3. ARCHIVE (SEDIT93) allows files to be written to floppys or the hard"
     PRINT " disk"
     PRINT "4. Please note that ARCHIVE will file all assigned data from BOTH the"
     PRINT " RSA and COULTER files."
     PRINT
     PRINT "If you wish to archive now, indicate the drive (A, B, C ...) on which"
     PRINT "the file should be recorded. If using a floppy, place a diskette in the drive"
     PRINT "Hit `Enter' to start Archive or to exit."
     DO
             INPUT "Transfer Drive (A,B,C...) "; A$
             IF A$ = "" THEN
                PRINT "NO DRIVE WAS DESIGNATED - NO TRANSFER FILE WRITTEN"
                BEEP                                    'new SEDIT 93
                SLEEP 3
                RETURN          ' MAIN MENU
             END IF
             A$ = LEFT$(UCASE$(A$), 1)
     LOOP WHILE A$ < "A" OR A$ > "I"
     TDV$ = A$
     B$ = "":   ' OLD MFN
     RSAflag% = TRUE: CLTRflag% = TRUE: Bothflag% = TRUE:   ' SETUP FOR BOTH
     GOSUB ReadIndexFiles:                   ' READ LN# & STATUS
    
     ' rem routine to save old SEDXFER.DAT as SEDXFER.OLD       AHE 8-93
     ON ERROR GOTO StartXfer
     OPEN "R", #2, TDV$ + ":SEDXFER.DAT"
     CLOSE #2
     OPEN "R", #2, TDV$ + ":SEDXFER.OLD"
     CLOSE #2

     KILL TDV$ + ":SEDXFER.OLD"
     NAME TDV$ + ":SEDXFER.DAT" AS TDV$ + ":SEDXFER.OLD"
     PRINT TDV$ + ":SEDXFER.DAT has been renamed " + TDV$ + ":SEDXFER.OLD"
     PRINT "Now writing new " + TDV$ + ":SEDXFER.DAT"
     SLEEP 3
StartXfer:
     ON ERROR GOTO 0
     OPEN "O", #2, TDV$ + ":SEDXFER.DAT"
     IF Rfiles% = 0 THEN GOTO CoulterArchive
     FOR Index% = 1 TO Rfiles%
             RSAlabNos$(Index%) = MID$(RSAlabNos$(Index%), 6, 1)
     NEXT Index%
     PRINT "ARCHIVING RSA DATA"
     GOSUB OpenRSA                              ' OPEN RSA
     FOR J% = 1 TO Rfiles%
          IF RSAlabNos$(J%) = "A" THEN
               GET #1, J%
               GOSUB FillBufRSA:                ' FILL ED BUFFER
               ED$(9) = LongDate$               ' Y2K Change to MM/DD/YYYY
               'above is is ok because data are to be written delimited
               A$ = ED$(1)
               IF A$ <> B$ THEN PRINT #2, "**" + A$: B$ = A$:   ' PRINT NEW MFN
               PRINT #2, ">>" + ED$(2) + "  " + ED$(3)
               PRINT ED$(2), ED$(3)
               FOR Index% = 4 TO 13
                    IF Index% = 10 THEN Index% = 11:   ' SKIP HOUR
                    PRINT #2, ED$(Index%)
               NEXT Index%
               FOR Index% = 14 TO 23
                    PRINT #2, Index% - 19; "  " + ED$(Index%)
               NEXT Index%
          END IF
     NEXT J%
     CLOSE #1

CoulterArchive:

     FOR Index% = 1 TO Cfiles%
          CLTlabNos$(Index%) = MID$(CLTlabNos$(Index%), 6, 1)
     NEXT Index%
     PRINT "ARCHIVING CLTR DATA"
     GOSUB OpenCLTR                          ' OPEN CLTR
     FOR J% = 1 TO Cfiles%
          IF CLTlabNos$(J%) = "A" THEN
               GET #1, J%
               GOSUB FillBufCLTR:                 ' FILL ED BUFFER
               ED$(9) = LongDate$               ' Y2K Change to MM/DD/YYYY
               'above is is ok because data are to be written ascii delimited
               A$ = ED$(1)
               IF A$ <> B$ THEN PRINT #2, "**" + A$: B$ = A$:   ' PRINT NEW MFN
               PRINT #2, ">>" + ED$(2) + "  " + ED$(3)
               PRINT ED$(2), ED$(3)
              
               FOR Index% = 4 TO 9
                    PRINT #2, ED$(Index%)
               NEXT Index%
              
               FOR Index% = 0 TO 15
                    IF VAL(ED$(Index% + 11)) <> 0 THEN
                         PRINT #2, ED$(11 + Index%) + "  " + ED$(27 + Index%)
                    END IF
               NEXT Index%
          END IF
     NEXT J%

     CLOSE
     OPEN "R", #1, "RSA.NDX", 9
     FIELD #1, 5 AS DUMMY$, 1 AS DISC$, 3 AS DUMMY2$
     OpnSamps% = 0
    
     FOR Index% = 1 TO Rfiles%
          IF RSAlabNos$(Index%) = "A" THEN
               GET #1, Index% + 1
               LSET DISC$ = "F"
               PUT #1, Index% + 1
          END IF
          IF RSAlabNos$(Index%) = "O" THEN OpnSamps% = OpnSamps% + 1
     NEXT Index%
    
     CLOSE
     OPEN "R", #2, "CLTR.NDX", 9
     FIELD #2, 5 AS DUMMY$, 1 AS DISC$, 3 AS DUMMY2$
    
     FOR Index% = 1 TO Cfiles%
          IF CLTlabNos$(Index%) = "A" THEN
               GET #2, Index% + 1
               LSET DISC$ = "F"
               PUT #2, Index% + 1
          END IF
          IF CLTlabNos$(Index%) = "O" THEN OpnSamps% = OpnSamps% + 1
     NEXT Index%
    
     CLOSE
     IF OpnSamps% = 0 THEN
          PRINT "All data are now FILED or DELETED. Enter 'K' to kill the temporary"
          PRINT "files on the RSA/COULTER disk. Any other key will leave them intact."
          DO
               A$ = (INKEY$)
          LOOP WHILE A$ = ""

          IF A$ = "k" OR A$ = "K" THEN
               OPEN "R", #1, "RSA.NDX", 9
               FIELD #1, 5 AS DISC$, 4 AS DUMMY$
               LSET DISC$ = "00000": PUT #1, 1
               CLOSE
               OPEN "R", #1, "CLTR.NDX", 9
               FIELD #1, 5 AS DISC$, 4 AS DUMMY$
               LSET DISC$ = "00000": PUT #1, 1
               CLOSE
               OpnSamps% = 0
          END IF
     END IF
PRINT "All assigned samples have been written to the TRANSFER DISK."
PRINT OpnSamps%; " samples remain open."
INPUT "Hit 'Enter' to return to main menu. ", B$
A$ = "B"
RETURN




'**************************
'***    SUBROUTINES     ***
'**************************

IsItAssigned:
'***************** Test for Previous Assign or Delete Status ****************
IF NOT IC% THEN
     B$ = MID$(RSAlabNos$(K%), 6, 1)
     C$ = RIGHT$(RSAlabNos$(K%), 3)
END IF
IF IC% THEN
     B$ = MID$(CLTlabNos$(K%), 6, 1)
     C$ = RIGHT$(CLTlabNos$(K%), 3)
END IF
NoGood% = FALSE
IF B$ = "*" OR B$ = "\" OR B$ = "O" THEN RETURN
PRINT "WARNING! You are attempting to open previously assigned or deleted record."
INPUT "If that's ok, type `YES' ", A$
IF A$ <> "YES" AND A$ <> "yes" THEN NoGood% = TRUE
RETURN

SetRSA:
'********************* LSET RSA.DAT *****************************************
     LSET QM$ = ED$(1)
     LSET QLabNo$ = ED$(2)
     LSET QTubeDia$ = ED$(3)
     LSET QSampID$ = ED$(4)
     LSET QProjID$ = ED$(5)
     LSET QCruiseID$ = ED$(6)
     LSET QReqBy$ = ED$(7)
     LSET QOperID$ = ED$(8)

'    Y2K - QDate$ come from ED$ element #9 and must be 8 characters
     LSET QDate$ = ED$(9)

     LSET QTime$ = ED$(10)
     LSET QSampWeight$ = ED$(11)
     LSET QCoarseWeight$ = ED$(12)
     LSET QSandWeight$ = ED$(13)
     FOR Index% = 0 TO 9
          LSET QPhiData$(Index%) = ED$(14 + Index%)
     NEXT Index%
RETURN

SetCLTR:
'********************* LSET CLTR.DAT ****************************************
     LSET QM$ = ED$(1)
     LSET QLabNo$ = ED$(2)
     LSET QTubeDia$ = ED$(3)
     LSET QSampID$ = ED$(4)
     LSET QProjID$ = ED$(5)
     LSET QCruiseID$ = ED$(6)
     LSET QReqBy$ = ED$(7)
     LSET QOperID$ = ED$(8)

' Y2K -  QDate$ = ED$ element 9
     LSET QDate$ = ED$(9)
     LSET QTime$ = ED$(10)
     FOR Index% = 0 TO 15
          LSET QMicronDia$(Index%) = ED$(11 + Index%)
          LSET QVolPcnt$(Index%) = ED$(27 + Index%)
     NEXT Index%
RETURN

FillBufRSA:
'******************* FILL ED BUFFER FROM RSA.DAT **************************
     ED$(1) = QM$
     ED$(2) = UCASE$(QLabNo$)                           'SEDIT 93 AHE
     ED$(3) = QTubeDia$
     ED$(4) = QSampID$
     ED$(5) = QProjID$
     ED$(6) = QCruiseID$
     ED$(7) = QReqBy$
     ED$(8) = QOperID$
     ED$(9) = QDate$
     ED$(10) = QTime$
     ED$(11) = QSampWeight$
     ED$(12) = QCoarseWeight$
     ED$(13) = QSandWeight$
    
     FOR Index% = 0 TO 9
          ED$(14 + Index%) = QPhiData$(Index%)
     NEXT Index%

     ' Y2K - change either format to mm/dd/yyyy format for transfer file
     ' check for old date format
     IF MID$(ED$(9), 3, 1) = "/" THEN           'date is mm/dd/yy format
        YY$ = "19" + RIGHT$(ED$(9), 2)          'make year 4 digit
        LongDate$ = LEFT$(ED$(9), 6)            'get the mm/dd/
        LongDate$ = LongDate$ + YY$             'LongDate now has mm/dd/yyyy format
     ELSE                                       'date is mmddyyyy format
        MM$ = LEFT$(QDate$, 2)                  'pull out the m,d,y
        DD$ = MID$(QDate$, 3, 2)
        YY$ = MID$(QDate$, 5, 4)
        LongDate$ = MM$ + "/" + DD$ + "/" + YY$ 'reformat as mm/dd/yyyy
     END IF


RETURN

FillBufCLTR:
'******************** FILL ED BUFFER FROM CLTR.DAT *************************
     ED$(1) = QM$
     ED$(2) = UCASE$(QLabNo$)                           'SEDIT 93 AHE
     ED$(3) = QTubeDia$
     ED$(4) = QSampID$
     ED$(5) = QProjID$
     ED$(6) = QCruiseID$
     ED$(7) = QReqBy$
     ED$(8) = QOperID$
    
' Y2K - date  
     ED$(9) = QDate$
     ED$(10) = QTime$
     FOR Index% = 0 TO 15
          ED$(11 + Index%) = QMicronDia$(Index%)
          ED$(27 + Index%) = QVolPcnt$(Index%)
     NEXT Index%
    
     ' Y2K - change either format to mm/dd/yyyy format for transfer file
     ' check for old date format
     IF MID$(ED$(9), 3, 1) = "/" THEN           'date is mm/dd/yy format
        YY$ = "19" + RIGHT$(ED$(9), 2)          'make year 4 digit
        LongDate$ = LEFT$(ED$(9), 6)            'get the mm/dd/
        LongDate$ = LongDate$ + YY$             'LongDate now has mm/dd/yyyy format
     ELSE                                       'date is mmddyyyy format
        MM$ = LEFT$(QDate$, 2)                  'pull out the m,d,y
        DD$ = MID$(QDate$, 3, 2)
        YY$ = MID$(QDate$, 5, 4)
        LongDate$ = MM$ + "/" + DD$ + "/" + YY$ 'reformat as mm/dd/yyyy
     END IF

RETURN

ReadIndexFiles:
'********** Open index files and read Lab#'s and Status ************

     PRINT "Reading status files.... Please wait"
     FLN$ = "CLTR": IF RSAflag% THEN FLN$ = "RSA"
     DO
          OPEN "R", #1, FLN$ + ".NDX", 9          'AHE 8-93
          FIELD #1, 5 AS DISC$, 4 AS DUMMY$
          GET #1, 1
          Nfiles% = VAL(DISC$)
          FIELD #1, 9 AS DISC$
          PRINT Nfiles%; " "; FLN$; " records found"
          PRINT "Reading..."
          IF FLN$ = "RSA" THEN Rfiles% = Nfiles%
          IF FLN$ = "CLTR" THEN Cfiles% = Nfiles%
          FOR Index% = 2 TO Nfiles% + 1
               GET #1, Index%
               IF FLN$ = "RSA" THEN
                    RSAlabNos$(Index% - 1) = DISC$
               ELSE CLTlabNos$(Index% - 1) = DISC$
               END IF
          NEXT Index%
          CLOSE
          DoAgain% = FALSE
          IF Bothflag% AND FLN$ = "RSA" THEN
               FLN$ = "CLTR"
               DoAgain% = TRUE
          END IF
     LOOP WHILE DoAgain%
     SLEEP 2
RETURN

OpenRSA:
'********************* OPEN AND FIELD RSA.DAT *********************************
     CLOSE #1
     OPEN "R", #1, "RSA.DAT", 181
     'Y2K Qdate$ is fielded as 8 character
     FIELD #1, 6 AS QM$, 5 AS QLabNo$, 3 AS QTubeDia$, 16 AS QSampID$, 16 AS QProjID$, 16 AS QCruiseID$, 16 AS QReqBy$, 16 AS QOperID$, 8 AS QDate$, 8 AS QTime$
     FIELD #1, 110 AS DUMMY$, 7 AS QSampWeight$, 7 AS QCoarseWeight$, 7 AS QSandWeight$, 5 AS QPhiData$(0), 5 AS QPhiData$(1), 5 AS QPhiData$(2), 5 AS QPhiData$(3), 5 AS QPhiData$(4), 5 AS QPhiData$(5), 5 AS QPhiData$(6), 5 AS QPhiData$(7), 5 AS  _
QPhiData$(8), 5 AS QPhiData$(9)
RETURN

OpenCLTR:
'********************* OPEN AND FIELD CLTR.DAT ********************************
     CLOSE #1
     OPEN "R", #1, "CLTR.DAT", 254
     ' Y2K date  only 8 digits
     FIELD #1, 6 AS QM$, 5 AS QLabNo$, 3 AS QTubeDia$, 16 AS QSampID$, 16 AS QProjID$, 16 AS QCruiseID$, 16 AS QReqBy$, 16 AS QOperID$, 8 AS QDate$, 8 AS QTime$
     FOR Index% = 0 TO 15
          FIELD #1, (Index% * 9 + 110) AS DUMMY$, 5 AS QMicronDia$(Index%), 4 AS QVolPcnt$(Index%)
     NEXT Index%
RETURN

DisplayStatus:
'************************DISPLAY/PRINT STATUS ********************************
     ' ENTER WITH FLN$, Index%, Prntr% AND PROPER .DAT OPEN
     row% = CSRLIN
     col% = POS(0)
     LOCATE row%, 1
     COLOR 15, 0
     PRINT Lastline$
     COLOR 0, 15
     Istr$ = STR$(Index%)
     Istr$ = SPACE$(5 - LEN(Istr$)) + Istr$
     IF FLN$ = "RSA" THEN
          PRINT Istr$ + ". " + "RSA      "; LEFT$(RSAlabNos$(Index%), 5);
          Lastline$ = Istr$ + ". " + "RSA      " + LEFT$(RSAlabNos$(Index%), 5)
          IF Prntr% THEN LPRINT "RSA      "; LEFT$(RSAlabNos$(Index%), 5);
     END IF

     IF FLN$ = "CLTR" THEN
          PRINT Istr$ + ". " + "COULTER  "; LEFT$(CLTlabNos$(Index%), 5);
          Lastline$ = Istr$ + ". " + "COULTER  " + LEFT$(CLTlabNos$(Index%), 5)
          IF Prntr% THEN LPRINT "COULTER  "; LEFT$(CLTlabNos$(Index%), 5);

     END IF
     IF FLN$ = "RSA" THEN
          ST$ = MID$(RSAlabNos$(Index%), 6, 1)    ' Mods next 20 lines
          Tdia$ = RIGHT$(RSAlabNos$(Index%), 3)   ' AHE 8-93
     END IF
     IF FLN$ = "CLTR" THEN
          ST$ = MID$(CLTlabNos$(Index%), 6, 1)
          Tdia$ = RIGHT$(CLTlabNos$(Index%), 3)
     END IF
     PRINT " "; Tdia$;
     IF Prntr% THEN LPRINT " "; Tdia$
     Lastline$ = Lastline$ + " " + Tdia$
     IF ST$ = "O" THEN
          PRINT " ### OPEN ###";
          Lastline$ = Lastline$ + " ### OPEN ###"
          IF Prntr% THEN LPRINT "  ### OPEN ###"
     END IF

     IF ST$ = "D" THEN
          PRINT "  DELETED";
          Lastline$ = Lastline$ + "  DELETED"
          IF Prntr% THEN LPRINT "  DELETED"
     END IF

     IF ST$ = "F" THEN
          PRINT "  FILED";
          Lastline$ = Lastline$ + "  FILED"
          IF Prntr% THEN LPRINT "  FILED"
     END IF

     IF ST$ = "A" THEN
          PRINT "  ASSIGNED TO ";
          Lastline$ = Lastline$ + "  ASSIGNED TO "
          IF Prntr% THEN LPRINT "  ASSIGNED TO ";
     END IF

     IF ST$ = "*" THEN
        PRINT CHR$(9); CHR$(9); "Tagged for assignment to "; MFN$;
        Lastline$ = Lastline$ + CHR$(9) + CHR$(9) + "Tagged for assignment to " + MFN$
     END IF
     IF ST$ = "\" THEN
        PRINT CHR$(9); CHR$(9); "Tagged for deletion";
        Lastline$ = Lastline$ + CHR$(9) + CHR$(9) + "Tagged for deletion"
     END IF
     IF ST$ = "^" THEN
        PRINT "  Tagged for re-openning";
        Lastline$ = Lastline$ + "  Tagged for re-openning"
     END IF
     IF ST$ = "A" THEN
          GET #1, Index%: A$ = QM$
          PRINT A$;
          Lastline$ = Lastline$ + A$
          IF Prntr% THEN LPRINT A$
     END IF
     COLOR 15, 0
RETURN

EditMenu:
'*************************** PRINT EDIT MENU *******************************
     PRINT "Commands:    ? Help    Display this menu"
     PRINT "             A Assign  Assign Master File Name"
     PRINT "             B Back    Backup to previous sample"
     PRINT "             D Delete  Delete this sample"
     PRINT "             E Edit    Edit or Examine DATA for this sample"
     PRINT "             O Open    Reopen TAGGED sample"
     PRINT "             Q Quit    MUST be done to exit session"
     PRINT "            `Esc'      Same as `Q'  to exit"
     PRINT "            `Enter'    Moves to next sample"
     PRINT "             PG UP     Move UP 20 Samples"
     PRINT "             PG DN     Move Down 20 Samples"
     PRINT "             ARROW     keys also scroll through samples"

RETURN

PrintFault:
     PRINT
     PRINT "Printer not responding. Check for Power, Paper, Etc."
     INPUT "Press `Enter' and retry print option.", C$
     PRINT
     BEEP
     RESUME MainMenu


'************************* END OF PROGRAM ***************************************

DEFSNG I-M
FUNCTION KeyCode% STATIC
  ' ************************************************
  ' **  Name:          KeyCode%                   **
  ' **  Type:          Function                   **
  ' **  Module:        KEYS.BAS                   **
  ' **  Language:      Microsoft QuickBASIC 4.00  **
  ' ************************************************
  '
  ' Returns a unique integer for any key pressed.
  '
  ' EXAMPLE OF USE:  k% = KeyCode%
  ' PARAMETERS:      (none)
  ' VARIABLES:       (none)
  ' MODULE LEVEL
  '   DECLARATIONS:  DECLARE FUNCTION KeyCode% ()
  '
        DO
            K$ = INKEY$
        LOOP UNTIL K$ <> ""
        KeyCode% = CVI(K$ + CHR$(0))

END FUNCTION

